private void DrawLine(Canvas canvas, int x, int y, int endX, int endY, Paint paint)
{
	if ((Math.abs(x - endX) > 5) || (Math.abs(y - endY) > 5))
	{
		mBrushForce = 0;
	}
	if (x == endX && y == endY)
	{
		DrawPoint(canvas, x, y, paint);
	}
	else if (x == endX)
	{
		for (int i = y; i < endY; i ++)
		{
			DrawPoint(canvas, x, i, paint);
		}
	}
	else if (y == endY)
	{
		for (int i = x; i < endX; i ++)
		{
			DrawPoint(canvas, i, y, paint);
		}
	}
	else
	{
		float dx = (float)(endX - x);
		float dy = (float)(endY - y);
		float step = mBitmapBrushWidth/8;
		if (Math.abs(dx) < Math.abs(dy))
		{
			float tmpx = x;
			
			dx = step*dx/Math.abs(dy);
			dy = step;
			if (endY < y)
			{
				for (int i = y; i > endY; i -= step)
				{
					DrawPoint(canvas, (int)tmpx, i, paint);
					tmpx += dx;
				}
			}
			else
			{
				for (int i = y; i < endY; i += step)
				{
					DrawPoint(canvas, (int)tmpx, i, paint);
					tmpx += dx;
				}
			}
		}
		else
		{
			float tmpy = y;
			
			dy = step*dy/Math.abs(dx);
			dx = step;
			if (endX < x)
			{
				for (int i = x; i > endX; i -= step)
				{
					DrawPoint(canvas, i, (int)tmpy, paint);
					tmpy += dy;
				}
			}	
			else
			{
				for (int i = x; i < endX; i += step)
				{
					DrawPoint(canvas, i, (int)tmpy, paint);
					tmpy += dy;
				}
			}
		}
	}
}


private void DrawPoint(Canvas canvas, int x, int y, Paint paint)
{
	if (mBitmapBrush != null)
	{
		int bw = mBitmapBrushWidth;
		int bh = mBitmapBrushHeight;
		
		if (mBrushForce < 0)
		{
			mBrushForce = 0;
		}
		else if (mBrushForce >= mBitmapBrushCols)
		{
			mBrushForce = mBitmapBrushCols - 1;
		}
		
		int indexR = mBrushForce;
		int indexC =  mRandomGenerator.nextInt( mBitmapBrushCols );
		int x0 = indexC*mBitmapBrushHeight;
		int y0 = indexR*mBitmapBrushWidth;

		if (mConfig_EnableFreeDrawingLayer)
		{
			canvas.drawBitmap(mBitmapBrush, new Rect(x0, y0, x0 + mBitmapBrushWidth, y0 + mBitmapBrushHeight), new Rect(x - mBitmapBrushWidth/2, y - mBitmapBrushHeight/2, x + mBitmapBrushWidth/2, y + mBitmapBrushHeight/2), paint);
		}
		else
		{
			// canvas.drawBitmap(mBitmapBrush, new Rect(x0, y0, x0 + mBitmapBrushWidth, y0 + mBitmapBrushHeight), new Rect(x - mBitmapBrushWidth/2, y - mBitmapBrushHeight/2, x + mBitmapBrushWidth/2, y + mBitmapBrushHeight/2), paint);
			
			int reg_w = mOverlayBitmap.getWidth();
			int reg_h = mOverlayBitmap.getHeight();
			
			int desy = y - mBitmapBrushHeight/2;
			for (int _y = y0; _y < y0 + bh; _y++)
			{
				int burshArrayIndex = x0 + _y*mBitmapBrush.getWidth();
				int desx = x - mBitmapBrushWidth/2;
				int des_index = desx + desy*reg_w;
				
				for (int _x = x0; _x < x0 + bw; _x++)
				{
					if (desx >=0 && desx < reg_w && desy >= 0 && desy < reg_h && mBrushArray[burshArrayIndex] > 0 && mMaskColorArray[des_index] == mBrushReplacementColor)
					{
						// canvas.drawPoint(x + _x - x0 - mBitmapBrushWidth/2, y + _y - y0 - mBitmapBrushHeight/2, paint);
						mMaskColorArray[des_index] = 0x0;
						mOverlayBitmap.setPixel(desx, desy, 0x0);
					}
					burshArrayIndex++;
					desx++;
					des_index++;
				}
				desy++;
			}
			// mOverlayBitmap.setPixels(mMaskColorArray, 0, reg_w, 0, 0, reg_w, reg_h);
		}
	}
	else
	{
		canvas.drawCircle(x, y, mBrushSize, paint);
	}
}

private void _SaveToLibrary(String libraryName, String fileBaseName, String bg)
{
	String name = null;
	try {
		java.io.InputStream bitmap = mContext.getAssets().open(bg);
		Bitmap bit=BitmapFactory.decodeStream(bitmap);
		bitmap.close();
		
		Bitmap drawable = bit.copy(Config.ARGB_8888,true);
		Canvas c = new Canvas(drawable);

		c.drawBitmap(mFinalBitmap, (drawable.getWidth() - mFinalBitmap.getWidth())/2, (drawable.getHeight() - mFinalBitmap.getHeight())/2, null);
		name = MyActivity.s_Activity.SaveBitmap(drawable, libraryName, fileBaseName, true);
	} catch (java.io.IOException e) {
		name = MyActivity.s_Activity.SaveBitmap(mFinalBitmap, libraryName, fileBaseName, true);
	}

	if (name != null)
	{
		String[] words = name.split("/");
		try
		{
			MediaStore.Images.Media.insertImage(MyActivity.s_Activity.getContentResolver(), name, words[words.length-1], words[words.length-1]);
		}
		catch(Exception E){}
	}
	
}


/**
	Draw:
	There're 3 layers:
	+ mOverlayCanvas (mOverlayBitmap): use for drawing mMaskBitmap as mask
	+ mFinalCanvas (mFinalBitmap): draw mDisplayBitmap as background, and mOverlayBitmap as mask
	+ canvas (screen): draw mFinalCanvas as background, and cursor over
**/
private void Draw(Canvas canvas)
{
	//draw background
	if (mDisplayBitmap == null || mMaskBitmap == null)
	{
		return;
	}
	
	//---------------------------------------------------------------------------
	//Draw to mOverlayBitmap
	//---------------------------------------------------------------------------		
	if (mRequestReset)
	{
		mRequestReset = false;
		mOverlayCanvas.drawBitmap(mMaskBitmap, 0, 0, null); //redraw mask if reset (on overlayCanvas)
	}
	
	if ((mHasPaint || mRequestReset) && mHasAlreadySetBrushColor) //if has request paint
	{
		mHasPaint = false;
		
		if (mHasSetBrushColor)
		{
			if (mConfig_EnableFreeDrawingLayer)
			{
				mBrush = new Paint();
				mBrush.setXfermode(null);
				mBrush.setColor(mBrushColor);
				ColorFilter filter = new LightingColorFilter(mBrushColor, 1);
				mBrush.setColorFilter(filter);
			}
			else
			{
				mBrush = new Paint();
				mBrush.setXfermode(new AvoidXfermode(mBrushReplacementColor, 0, AvoidXfermode.Mode.TARGET));
				mBrush.setColor(Color.TRANSPARENT);
			}
			mHasSetBrushColor = false;
		}
		
		
		// System.out.println("************* " + mX + " " + mY + "  ----  " + mLastX + " " + mLastY);
		int x = mX;
		int y = mY;
		int lastX = mLastX;
		int lastY = mLastY;
		if (lastX == INF)
		{
			DrawPoint(mOverlayCanvas, mX, mY, mBrush);
			lastX = mX;
			lastY = mY;
		}
		else
		{
			if (lastX == x && lastY == y)
			{
				DrawPoint(mOverlayCanvas, x, y, mBrush);
			}
			else
			{
				int dx = Math.abs(lastX - x);
				int dy = Math.abs(lastY - y);
				if (dx < MyActivity.s_Activity.mDrawingViewX && dy < MyActivity.s_Activity.mDrawingViewY)
				{
					// System.out.println(lastX + " " + lastY + " " + x + " " + y);
					DrawLine(mOverlayCanvas, lastX, lastY, x, y, mBrush);
					lastX = x;
					lastY = y;
				}
				else
				{
					lastX = INF;
					lastY = INF;
				}
			}
		}
		mLastX = lastX;
		mLastY = lastY;
	}
	
	//---------------------------------------------------------------------------
	//Draw to mFinalBitmap
	//---------------------------------------------------------------------------		
	mFinalCanvas.drawBitmap(mDisplayBitmap, 0, 0, null); //redraw background
	mFinalCanvas.drawBitmap(mOverlayBitmap,0,0,null);
	
	//---------------------------------------------------------------------------
	//Draw to screen
	//---------------------------------------------------------------------------	
	canvas.drawBitmap(mFinalBitmap, 0, 0, null);
	
	// check if has request save
	if (mHasRequestSaveImage) 
	{
		mHasRequestSaveImage = false;
		_SaveToLibrary(mRequestSaveImageFolder, mRequestSaveImageName, mRequestSaveBGImageName);
		MyActivity.CallLua("bridge_DrawingWithMask_SaveToLibraryCompleted");
	}
}