/**
	public void SetCustomBitmapBrush(String bitmapURL, int rows, int cols)
	public float GetCompletionPercentage()
	public void SetBrushColor(int color)
	public void SetBrushSize(int size)
	public void Reset()
	public void EnableTouch(boolean val)
	public void SetDisplayBitmap(String url)
	public void SetMaskBitmap(String url)
	public void SaveToLibrary(String libraryName, String fileBaseName)
	public void SetFlags(int flag, boolean val)
**/

public void SetCustomBitmapBrush(String bitmapURL, int rows, int cols)
{
	if (bitmapURL == null)
	{
		mBrushArray =  null;
	}
	else
	{
		try
		{
			java.io.InputStream bitmap = mContext.getAssets().open(bitmapURL);
			Bitmap bit=BitmapFactory.decodeStream(bitmap);
			bitmap.close();
			mBitmapBrush = bit;
			
			mBrushArray = new int[mBitmapBrush.getWidth() * mBitmapBrush.getHeight()];
			mBitmapBrush.getPixels(mBrushArray, 0, mBitmapBrush.getWidth(), 0, 0, mBitmapBrush.getWidth(), mBitmapBrush.getHeight());
			mBitmapBrushCols = cols;
			mBitmapBrushRows = rows;
			mBitmapBrushWidth = mBitmapBrush.getWidth()/mBitmapBrushCols;
			mBitmapBrushHeight = mBitmapBrush.getHeight()/mBitmapBrushRows;
			
			for (int i = 0; i < mBrushArray.length; i++)
			{
				if (((mBrushArray[i]>>24)&0xFF) > 0)
				{
					mBrushArray[i] = 1;
				}
				else
				{
					mBrushArray[i] = 0;
				}
			}
		}
		catch (Exception E){
			E.printStackTrace();
		}
	}
}

public float GetCompletionPercentage()
{
	int samePixelCount = 0;
	int width = mFinalBitmap.getWidth();
	int height = mFinalBitmap.getHeight();
	int pixelCount = 0;
	
	int[] pixels1 = new int[width*height];
	int[] pixels2 = new int[width*height];
	mFinalBitmap.getPixels(pixels1,0,width,0,0,width,height);
	mDisplayBitmap.getPixels(pixels2,0,width,0,0,width,height);
	for(int i = 0;  i<width*height; i++)
	{
		int color1 = pixels1[i];
		int color2 = pixels2[i];
		if(Color.alpha(color1)>=1 && Color.alpha(color2) >= 1)
		{
			pixelCount++;
			if(color1 == color2)
				samePixelCount++;
		}
	}
	return (float)samePixelCount/pixelCount * 100;
}

public void SetBrushColor(int color, int maskColor)
{
	mHasSetBrushColor = true;
	mHasAlreadySetBrushColor = true;
	mBrushReplacementColor = (maskColor&0x00FFFFFF) | (0xFF000000);
	mBrushColor = (color&0x00FFFFFF) | (0xFF000000);
}

public void SetBrushSize(int size)
{
	mBrushSize = size;
	mBrush.setStrokeWidth(1);
}

public void Reset()
{
	mRequestReset = true;
}

public void EnableTouch(boolean val)
{
	mEnableTouch = val;
}

public void SetDisplayBitmap(String url)
{
	try {
		java.io.InputStream bitmap = mContext.getAssets().open(url);
		Bitmap bit=BitmapFactory.decodeStream(bitmap);
		bitmap.close();
		mDisplayBitmap = bit;
		mDisplayBitmap.setHasAlpha(true);
		mFinalBitmap = bit.copy(Config.ARGB_8888,true);
		mFinalCanvas = new Canvas(mFinalBitmap);
	} catch (java.io.IOException e) {
		e.printStackTrace();
	}
}

public void SetMaskBitmap(String url)
{
	try {
		java.io.InputStream bitmap = mContext.getAssets().open(url);
		Bitmap bit=BitmapFactory.decodeStream(bitmap);
		bitmap.close();
		mMaskBitmap = bit;
		mOverlayBitmap = bit.copy(Config.ARGB_8888, true);  
		mOverlayCanvas = new Canvas(mOverlayBitmap);  
		
		// create a temp argb buffer
		mMaskColorArray = new int[mMaskBitmap.getWidth() * mMaskBitmap.getHeight()];
		mMaskBitmap.getPixels(mMaskColorArray, 0, mMaskBitmap.getWidth(), 0, 0, mMaskBitmap.getWidth(), mMaskBitmap.getHeight());
	} catch (java.io.IOException e) {
		e.printStackTrace();
	}
}

public void SaveToLibrary(String libraryName, String fileBaseName, String BackgroundURL)
{
	mHasRequestSaveImage = true;
	mRequestSaveImageFolder = libraryName;
	mRequestSaveImageName = fileBaseName;
	mRequestSaveBGImageName = BackgroundURL;
}

public void SetFlags(int flag, boolean val)
{
	if ((flag&DRAWINGWITHMASKVIEW_FLAG_ENABLE_FREE_DRAWING) != 0)
	{
		mConfig_EnableFreeDrawingLayer = val;
		mHasSetBrushColor = true;
	}
	if ((flag&DRAWINGWITHMASKVIEW_FLAG_SIMULATE_SPRAY) != 0)
	{
		mConfig_SimulateSpray = val;
	}
	if ((flag&DRAWINGWITHMASKVIEW_FLAG_DISABLE_FINGER) != 0)
	{
		mConfig_DisableFinger = val;
	}
	if ((flag&DRAWINGWITHMASKVIEW_FLAG_DISABLE_SPEN_TOUCH) != 0)
	{
		mConfig_DisableSpenTouch = val;
	}
}