#if USE_BLUETOOTH
/********************************************************************
Bluetooth interface
/********************************************************************/
static void Bluetooth_Init()
{
	s_Activity.SendMessge(MESSAGE_BT_INIT_BLUETOOTH);
}

static void Bluetooth_SearchDevice()
{
	s_Activity.SendMessge(MESSAGE_BT_SEARCH_DEVICE);
}

static void Bluetooth_StopSearchDevice()
{
	s_Activity.SendMessge(MESSAGE_BT_STOP_SEARCH_DEVICE);
}

static void Bluetooth_Connect(String device)
{
	s_Activity.mRequestedConnectedDevice = device;
	s_Activity.SendMessge(MESSAGE_BT_CONNECT);
}

static void Bluetooth_Close()
{
	s_Activity.Bluetooth_OnDestroy();
}

static void Bluetooth_StartListening()
{
	s_Activity.SendMessge(MESSAGE_BT_LISTEN);
}

static void Bluetooth_ResetService()
{
	s_Activity.SendMessge(MESSAGE_BT_RESET_SERVICE);
}

static int Bluetooth_GetServiceState()
{
	return s_Activity.GetBluetoothServiceState();
}

static String Bluetooth_GetDeviceName()
{
	return s_Activity.GetDeviceName();
}

static void Bluetooth_SendMessage(String msg)
{
	s_Activity.SendMessage(msg);
}

static String Bluetooth_GetPairedDevices()
{
	return s_Activity._Bluetooth_GetPairedDevices();
}
/********************************************************************
Bluetooth implementation
/********************************************************************/
public static String DEVICE_NAME = "device_name";
public static String DEVICE_ADDR = "device_addr";
public static final String TOAST = "toast";

private String mDeviceName = "";
private String mConnectedDeviceName = "";
private String mConnectedDeviceAddr = "";

private BluetoothChatService mChatService = null;

// Local Bluetooth adapter
private BluetoothAdapter mBluetoothAdapter = null;
private boolean mIsBluetoothSupported = false;

ArrayList<String> m_DeviceList;

String mRequestedConnectedDevice;

boolean mFixedDuplicateBTCompletedSignal = false;


private void Bluetooth_OnCreate()
{
	mIsBluetoothSupported = false;
	m_DeviceList = new ArrayList<String>();
}

private void _Bluetooth_Init()
{
	mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
	if (mBluetoothAdapter == null) 
	{
		String param[] = {"errorCode", "1", "msg", "Bluetooth is not supported"};
		MyActivity.DispatchLua("gBluetoothDriver","bridge_Bluetooth_Init_Completed", param);
		return;
	}
	mDeviceName = mBluetoothAdapter.getName();
	if(mDeviceName == null)
	{
		mDeviceName = mBluetoothAdapter.getAddress();
	}
	mIsBluetoothSupported = true;
	
	
	if (!mBluetoothAdapter.isEnabled()) 
	{
		Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
		startActivityForResult(enableIntent, REQUEST_ENABLE_BT);
	}
	else 
	{
		setupChat();
		String param[] = {"errorCode", "0", "msg", "Enable bluetooth successfully"};
		MyActivity.DispatchLua("gBluetoothDriver","bridge_Bluetooth_Init_Completed", param);
	}
}

private void _Bluetooth_EnsureDiscoverable() 
{
	if (!mIsBluetoothSupported) return;
	if (mBluetoothAdapter.getScanMode() !=
		BluetoothAdapter.SCAN_MODE_CONNECTABLE_DISCOVERABLE) {
		Intent discoverableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_DISCOVERABLE);
		discoverableIntent.putExtra(BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION, 300);
		startActivityForResult(discoverableIntent, REQUEST_ENABLE_DISCOVERY_BT);
	}
	else
	{
		this.setupChat();
		if (mChatService != null)
		{
			this.mChatService.listen();
		}
		String param[] = {"errorCode", "0", "msg", "Ensure Discoverable successfully"};
		MyActivity.DispatchLua("gBluetoothDriver","bridge_Bluetooth_Ensure_Discoverable_Completed", param);
	}
}

public String _Bluetooth_GetPairedDevices()
{
	String re = "";
	if (mIsBluetoothSupported)
	{
		Set<BluetoothDevice> pairedDevices = mBluetoothAdapter.getBondedDevices();
		if (pairedDevices.size() > 0) 
		{
			for (BluetoothDevice device : pairedDevices) 
			{
				re = re + device.getName() + "\n" + device.getAddress() + "\n";
			}
		} 
	}
	return re;
}

private void _Bluetooth_SearchDevice()
{
	m_DeviceList = new ArrayList<String>();
	
	Set<BluetoothDevice> pairedDevices = mBluetoothAdapter.getBondedDevices();
	// If there are paired devices, add each one to the ArrayAdapter
	if (pairedDevices.size() > 0) 
	{
		for (BluetoothDevice device : pairedDevices) 
		{
			LogI("[bluetooth] paired: " + device.getName() + " - " + device.getAddress());
			m_DeviceList.add("paired");
			m_DeviceList.add(device.getName());
			m_DeviceList.add(device.getAddress());
		}
	} 
	
	mFixedDuplicateBTCompletedSignal = false;
	if (mBluetoothAdapter.isDiscovering()) 
	{
		mFixedDuplicateBTCompletedSignal = true;
		mBluetoothAdapter.cancelDiscovery();
	}
	IntentFilter filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
	this.registerReceiver(mReceiver, filter);
	filter = new IntentFilter(BluetoothAdapter.ACTION_DISCOVERY_FINISHED);
	this.registerReceiver(mReceiver, filter);
	mBluetoothAdapter.startDiscovery();
}

private void _Bluetooth_StopSearchDevice()
{
	try
	{
		this.unregisterReceiver(mReceiver);
	}
	catch (Exception E){}
	if (mBluetoothAdapter.isDiscovering()) 
	{
		mBluetoothAdapter.cancelDiscovery();
	}
	else
	{
		_Bluetooth_SendListOfDiscoveredDevices(true);
	}
}

// The BroadcastReceiver that listens for discovered devices and
// changes the title when discovery is finished
private final BroadcastReceiver mReceiver = new BroadcastReceiver() 
{
	@Override
	public void onReceive(Context context, Intent intent) 
	{
		String action = intent.getAction();
		// When discovery finds a device
		if (BluetoothDevice.ACTION_FOUND.equals(action)) 
		{
			// Get the BluetoothDevice object from the Intent
			BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
			// If it's already paired, skip it, because it's been listed already
			if (device.getBondState() != BluetoothDevice.BOND_BONDED) 
			{
				String name = device.getName() ;
				String addr = device.getAddress() ;
				LogI("[bluetooth] new   :" + name + " - " + addr);
				boolean existed = false;
				for(int i = 0; i < m_DeviceList.size(); i+=3)
				{
					if (name.compareTo(m_DeviceList.get(i + 1)) == 0 && addr.compareTo(m_DeviceList.get(i+2)) == 0)
					{
						existed = true;
						break;
					}
				}
				if (!existed)
				{
					m_DeviceList.add("new");
					m_DeviceList.add(name);
					m_DeviceList.add(addr);
				}
			}
			_Bluetooth_SendListOfDiscoveredDevices(false);
		} 
		else if (BluetoothAdapter.ACTION_DISCOVERY_FINISHED.equals(action))
		{
			if (mFixedDuplicateBTCompletedSignal)
			{
				mFixedDuplicateBTCompletedSignal = false;
			}
			else
			{
				LogI("[bluetooth] BluetoothAdapter.ACTION_DISCOVERY_FINISHED");
				_Bluetooth_SendListOfDiscoveredDevices(true);
			}
		}
	}
};

private void _Bluetooth_SendListOfDiscoveredDevices(boolean finished)
{
	// System.out.println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
	String str = "";
	for (int i = 0; i < m_DeviceList.size(); i++)
	{	
		str += m_DeviceList.get(i) + "\n";
	}
	String param[] = {"finished", "false", "devices", str};
	if (finished)
	{
		param[1] = "true";
	}
	MyActivity.DispatchLua("gBluetoothDriver","bridge_Bluetooth_Discovered_Devices", param);
}

private void Bluetooth_OnResume()
{
	if (!mIsBluetoothSupported) return;
	// Performing this check in onResume() covers the case in which BT was
	// not enabled during onStart(), so we were paused to enable it...
	// onResume() will be called when ACTION_REQUEST_ENABLE activity returns.
	if (mChatService != null) {
		// Only if the state is STATE_NONE, do we know that we haven't started already
		if (mChatService.getState() == BluetoothChatService.STATE_NONE) {
		  // Start the Bluetooth chat services
		  mChatService.start();
		}
	}
}

private void Bluetooth_OnDestroy()
{
	if (!mIsBluetoothSupported) return;
	if (mBluetoothAdapter != null) 
	{
		if (mBluetoothAdapter.isDiscovering()) 
		{
			mBluetoothAdapter.cancelDiscovery();
		}
	}

	// Unregister broadcast listeners
	try
	{
		this.unregisterReceiver(mReceiver);
	}
	catch(Exception E) 
	{}
	
	if (mChatService != null) mChatService.stop();
}

private void _ConnectDevice(String address, boolean secure) 
{
	LogI("[bluetooth] Request connect device" + address);
	// Get the BluetoothDevice object
	BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);
	// Attempt to connect to the device
	mChatService.connect(device, secure);
}

public void SendMessage(String message) {
	if (!mIsBluetoothSupported) return;
	
	// Check that we're actually connected before trying anything
	if (mChatService.getState() != BluetoothChatService.STATE_CONNECTED) {
		LogI("[bluetooth] Bluetooth is not connected");
		return;
	}

	// Check that there's actually something to send
	if (message.length() > 0) {
		message = message + "\n"; //sometimes, some message are sent at the same time, so they're put together into 1 message, separated by \n
		// Get the message bytes and tell the BluetoothChatService to write
		byte[] send = message.getBytes();
		mChatService.write(send);
	}
}

// The Handler that gets information back from the BluetoothChatService
private void Bluetooth_HandleMessage(Message msg)
{
	switch (msg.what) 
	{
		case MESSAGE_BT_INIT_BLUETOOTH:
			this._Bluetooth_Init();
			break;
		
		case MESSAGE_BT_SEARCH_DEVICE:
			this._Bluetooth_SearchDevice();
			break;
			
		case MESSAGE_BT_STOP_SEARCH_DEVICE:
			this._Bluetooth_StopSearchDevice();
			break;
			
		case MESSAGE_BT_CONNECT:
			_ConnectDevice(this.mRequestedConnectedDevice, false);
			break;
 
		case MESSAGE_BT_START_SERVICE:
			this.setupChat();
			break;
		
		case MESSAGE_BT_RESET_SERVICE:
			if (mChatService != null)
			{
				mChatService.stop();
				mChatService.start();
			}
			break;
			
		case MESSAGE_BT_LISTEN:
			_Bluetooth_EnsureDiscoverable();
			break;
			
		case MESSAGE_BT_STATE_CHANGE:
			Log("[Bluetooth] MESSAGE_BT_STATE_CHANGE: " + msg.arg1);
			switch (msg.arg1) 
			{
				case BluetoothChatService.STATE_CONNECTED:
					String param[] = {"name", mConnectedDeviceName, "addr", mConnectedDeviceAddr};
					MyActivity.DispatchLua("gBluetoothDriver","bridge_Bluetooth_Connected", param);
					
					// Toast.makeText(getApplicationContext(), "Connected", Toast.LENGTH_SHORT).show();
					break;
				case BluetoothChatService.STATE_CONNECTING:
					// setStatus(R.string.title_connecting);
					break;
				case BluetoothChatService.STATE_LISTEN:
				case BluetoothChatService.STATE_NONE:
					// setStatus(R.string.title_not_connected);
					break;
			}
			break;
		
		case MESSAGE_BT_CONNECT_FAIL:
			MyActivity.DispatchLua("gBluetoothDriver","bridge_Bluetooth_Connected_Fail");
			break;
			
		case MESSAGE_BT_CONNECT_LOST:
			MyActivity.DispatchLua("gBluetoothDriver","bridge_Bluetooth_Connected_Lost");
			break;
			
		case MESSAGE_BT_DEVICE_NAME:
			mConnectedDeviceName = msg.getData().getString(DEVICE_NAME);
                // Toast.makeText(getApplicationContext(), "Connected to "
                               // + mConnectedDeviceName, Toast.LENGTH_SHORT).show();
			mConnectedDeviceAddr = msg.getData().getString(DEVICE_ADDR);
			break;
		case MESSAGE_BT_TOAST:
			// Toast.makeText(getApplicationContext(), msg.getData().getString(TOAST), Toast.LENGTH_SHORT).show();
			break;
		case MESSAGE_BT_READ:
			byte[] readBuf = (byte[]) msg.obj;
			String readMessage = new String(readBuf, 0, msg.arg1);
			String param[] = {"msg", readMessage};
			MyActivity.DispatchLua("gBluetoothDriver","bridge_Bluetooth_OnReceive", param);
			// Toast.makeText(getApplicationContext(), "read " + readMessage, Toast.LENGTH_SHORT).show();
			break;
		case MESSAGE_BT_WRITE:
			// byte[] writeBuf = (byte[]) msg.obj;
			// String writeMessage = new String(writeBuf);
			// Toast.makeText(getApplicationContext(), "write " + writeMessage, Toast.LENGTH_SHORT).show();
			break;
		}
}

private void Bluetooth_OnActivityResult(int requestCode, int resultCode, Intent data) {
	switch (requestCode) 
	{
		// When the request to enable Bluetooth returns
		case REQUEST_ENABLE_BT:
			if (resultCode == Activity.RESULT_CANCELED) 
			{
				String param[] = {"errorCode", "1", "msg", "User did not enable Bluetooth or an error occurred"};
				MyActivity.DispatchLua("gBluetoothDriver","bridge_Bluetooth_Init_Completed", param);
			} 
			else 
			{
				this.setupChat();
				String param[] = {"errorCode", "0", "msg", "Enable bluetooth successfully"};
				MyActivity.DispatchLua("gBluetoothDriver","bridge_Bluetooth_Init_Completed", param);
			}
			break;
		case REQUEST_ENABLE_DISCOVERY_BT:
			if (resultCode == Activity.RESULT_CANCELED)
			{
				String param[] = {"errorCode", "1", "msg", "User did not enable Bluetooth DISCOVERY or an error occurred"};
				MyActivity.DispatchLua("gBluetoothDriver","bridge_Bluetooth_Ensure_Discoverable_Completed", param);
			}
			else
			{
				this.setupChat();
				if (mChatService != null)
				{
					this.mChatService.listen();
				}
				String param[] = {"errorCode", "0", "msg", "Ensure Discoverable successfully"};
				MyActivity.DispatchLua("gBluetoothDriver","bridge_Bluetooth_Ensure_Discoverable_Completed", param);
			}
	}
}

private void setupChat() {
	// Initialize the BluetoothChatService to perform bluetooth connections
	if (mChatService == null)
	{
		mChatService = new BluetoothChatService(this, m_Handler);
	}
	mChatService.stop();
	mChatService.start();
}

public int GetBluetoothServiceState()
{
	if (mChatService != null)
	{
		return mChatService.getState();
	}
	else
	{
		return -1;
	}
}

public String GetDeviceName()
{
	return this.mDeviceName;
}

#endif