#if USE_FACEBOOK

private Session.StatusCallback statusCallback = new SessionStatusCallback();
private WebDialog dialog;
private String dialogAction;
private Bundle dialogParams;
// private boolean gameLaunchedFromDeepLinking = false;
private GraphUser currentFBUser;
private boolean fb_loggedIn = false;
private boolean fb_isResumed = false;
private List<GraphUser> friends;

private String mPendingPostScore = "0";
private boolean pendingPost = false;
private boolean hasRequestPublishPermission = true;

private static final String FRIENDS_KEY = "friends";
private static final String CURRENT_FB_USER_KEY = "current_fb_user";
private static final String LOGGED_IN_KEY = "logged_in";
private static final List<String> PERMISSIONS = Arrays.asList("publish_actions");


private class SessionStatusCallback implements Session.StatusCallback {
    @Override
    public void call(Session session, SessionState state, Exception exception) {
    	LogI("[FB] SessionStatusCallback " + exception);
        FB_UpdateView();
        session.isOpened();
        {
	        if(s_Activity.getHasRequestPublishPermission() == false && s_Activity.hasGrantedPublishPermission() == false)
	        {
	        	s_Activity.requestPublishPermission();
	        }
	    }
    }
}

void FB_OnCreate(Bundle savedInstanceState) {
	LogI("[FB] Aplication sign " + CHelper.GetSign((Context)this));
	Session session = Session.getActiveSession();
	if (session == null) {
		if (savedInstanceState != null) {
			LogI("[FB] Restore session");
			session = Session.restoreSession(this, null, statusCallback, savedInstanceState);
		}
		if (session == null) {
			LogI("[FB] Create new session");
			session = new Session(this);
		}
		Session.setActiveSession(session);
		if (session.getState().equals(SessionState.CREATED_TOKEN_LOADED)) {
			session.openForRead(new Session.OpenRequest(this).setCallback(statusCallback));
		}
	}

	// load user info
	if (savedInstanceState != null) 
	{
		boolean loggedInState = savedInstanceState.getBoolean(this.getLoggedInKey(), false);
		this.setLoggedIn(loggedInState);
		
		LogI("[FB] Load user info from savedInstanceState " + this.isLoggedIn() + " " + this.getCurrentFBUser() + " " + this.getFriends());

		if (this.isLoggedIn() && ( this.getFriends() == null || (this.getCurrentFBUser() == null)) ) 
		{
			try {
				// currentFBUser
				String currentFBUserJSONString = savedInstanceState.getString(this.getCurrentFbUserKey());
				if (currentFBUserJSONString != null) {
					GraphUser currentFBUser = GraphObject.Factory.create(new JSONObject(currentFBUserJSONString), GraphUser.class);
					this.setCurrentFBUser(currentFBUser);
	 			}

				// friends
				ArrayList<String> friendsJSONStringArrayList = savedInstanceState.getStringArrayList(this.getFriendsKey());
				if (friendsJSONStringArrayList != null) 
				{
					ArrayList<GraphUser> friends = new ArrayList<GraphUser>();
					Iterator<String> friendsJSONStringArrayListIterator = friendsJSONStringArrayList.iterator();
					while (friendsJSONStringArrayListIterator.hasNext()) {
						friends.add(GraphObject.Factory.create(new JSONObject(friendsJSONStringArrayListIterator.next()), GraphUser.class));
					}
					this.setFriends(friends);
	 			}
	 		} catch (Exception e) {
	 			LogI("[FB] FB_OnCreate - Fail to load user data from savedInstanceState " + e.toString());
	 		}
		}
	}
}

void FB_OnActivityResult(int requestCode, int resultCode, Intent data)
{
	LogI("[FB] FB_OnActivityResult " + requestCode + " " + resultCode);
	Session.getActiveSession().onActivityResult(this, requestCode, resultCode, data);
}

void FB_OnResume()
{
	LogI("[FB] FB_OnResume");
	Session.getActiveSession().addCallback(statusCallback);
	fb_isResumed = true;

	if (this.getCurrentFBUser() != null) {
		LogI("[FB] FB_OnResume - user -" + this.getCurrentFBUser());
		Uri target = getIntent().getData();
		//String data = getIntent().getExtras().getString("data");
		LogI("[FB] FB_OnResume - Check deep link " + target);
		//LogI("[FB] FB_OnResume - data " + data);

		if (target != null) {
			String graphRequestIDsForSendingUser = target.getQueryParameter("request_ids");
			String feedPostIDForSendingUser = target.getQueryParameter("challenge_brag");

			LogI("[FB] FB_OnResume graphRequestIDsForSendingUser " + graphRequestIDsForSendingUser);
			LogI("[FB] FB_OnResume feedPostIDForSendingUser " + feedPostIDForSendingUser);

			if (graphRequestIDsForSendingUser != null) {
				String [] graphRequestIDsForSendingUsers = graphRequestIDsForSendingUser.split(",");
				String graphRequestIDForSendingUser = graphRequestIDsForSendingUsers[graphRequestIDsForSendingUsers.length-1];

				
				LogI("[FB] Resume from request " + graphRequestIDForSendingUser);
				MyActivity.DispatchLua("gGameInstance","bridge_FbEvent", new String[]{
					"requestID", graphRequestIDForSendingUser, 
					"service", "FB_OnResumeFromRequest"
				});

				Request deleteFBRequestRequest = new Request(Session.getActiveSession(),
					graphRequestIDForSendingUser + "_" + this.getCurrentFBUser().getId(),
					new Bundle(),
					HttpMethod.DELETE,
					new Request.Callback() {
						@Override
						public void onCompleted(Response response) {
							FacebookRequestError error = response.getError();
							if (error != null) {
								LogI("[FB] Deleting consumed Request failed: " + error.getErrorMessage());
							} else {
								LogI("[FB] Consumed Request deleted");
							}
						}
					});
				Request.executeBatchAsync(deleteFBRequestRequest);
			}
			else if (feedPostIDForSendingUser != null) { //receive challenge from friend
				LogI("[FB] Resume from feed " + feedPostIDForSendingUser);
				MyActivity.DispatchLua("gGameInstance","bridge_FbEvent", new String[]{
					"senderID", feedPostIDForSendingUser + "", 
					"service", "FB_OnResumeFromFeedPost"
				});
			}
		}
	}
	com.facebook.Settings.publishInstallAsync(this, this.getString(R.string.fb_app_id));
}

void FB_OnPause()
{
	LogI("[FB] FB_OnPause");
	fb_isResumed = false;
	Session.getActiveSession().removeCallback(statusCallback);
}

void FB_OnSaveInstanceState(Bundle outState)
{
	LogI("[FB] FB_OnSaveInstanceState");
	Session session = Session.getActiveSession();
	Session.saveSession(session, outState);

	//save user info
	if (this.getCurrentFBUser() != null) {
		outState.putString(	this.getCurrentFbUserKey(),
							this.getCurrentFBUser().getInnerJSONObject().toString());
	}

	// Save the logged-in state
	outState.putBoolean(this.getLoggedInKey(), this.isLoggedIn());

	// Save the logged-in user's list of friends
	if (this.getFriends() != null) {
		outState.putStringArrayList(this.getFriendsKey(), this.getFriendsAsArrayListOfStrings());
	}
}


void FB_OnDestroy()
{
	LogI("[FB] FB_OnDestroy");
}

void FB_UpdateView()
{
	if (fb_isResumed)
	{
		Session session = Session.getActiveSession();
		if (session.isOpened())
		{
			LogI("[FB] Session is open");
			if  (this.isLoggedIn())
			{
				FB_DispatchEvent("FB_Login");
			}
			else
			{
				if (s_Activity.hasGrantedPublishPermission())				
				{
					fetchUserInformation();
				}
			}
		}
		else if (session.isClosed())
		{
			LogI("[FB] Session is closed");
			if (this.isLoggedIn())
			{
				s_Activity.setFriends(null);
				s_Activity.setCurrentFBUser(null);
				s_Activity.setLoggedIn(false);
			}
			FB_DispatchEvent("FB_Logout");
		}

		if (session.isOpened())
		{
			tokenUpdated();
		}
	}
}

void fetchUserInformation()
{
	final Session session = Session.getActiveSession();

	if (session != null && session.isOpened()) {
		LogI("[FB] Get list of friends");
	    // If the session is open, make an API call to get user information required for the app

		// Get the user's list of friends
		Request friendsRequest = Request.newMyFriendsRequest(session, new Request.GraphUserListCallback() {
			@Override
			public void onCompleted(List<GraphUser> users, Response response) {
				FacebookRequestError error = response.getError();
				if (error != null) {
					LogI("[FB] fail to load friend" + error);
				} 
				else if (session == Session.getActiveSession()) {
	                // Set the friends attribute
					s_Activity.setFriends(users);					
					LogI("[FB] loaded friends " + s_Activity.getFriendsAsArrayListOfStrings().toString());
	            }
	        }
	    });
		Bundle params = new Bundle();
		params.putString("fields", "name");
		// params.putString("fields", "name,first_name,last_name");
		friendsRequest.setParameters(params);

		// Get current logged in user information
		LogI("[FB] Get user logged information");
		Request meRequest = Request.newMeRequest(session, new Request.GraphUserCallback() {
			@Override
			public void onCompleted(GraphUser user, Response response) {
				FacebookRequestError error = response.getError();
				if (error != null) {
					LogI("[FB] fail to load user" + error);
				} else if (session == Session.getActiveSession()) {
					// Set the currentFBUser attribute
					LogI("[FB] loaded loged user " + user);
					s_Activity.setCurrentFBUser(user);
				}
			}
		});

		// Create a RequestBatch and add a callback once the batch of requests completes
		// RequestBatch requestBatch = new RequestBatch(friendsRequest, meRequest);
		RequestBatch requestBatch = new RequestBatch(meRequest);
		requestBatch.addCallback(new RequestBatch.Callback() {
			@Override
	        public void onBatchCompleted(RequestBatch batch) {
				// if ( s_Activity.getCurrentFBUser() != null && s_Activity.getFriends() != null ) {
				if ( s_Activity.getCurrentFBUser() != null) {
					LogI("[FB] load user information successful");
					s_Activity.setLoggedIn(true);
					FB_DispatchEvent("FB_Login");
	            } else {
	                LogI("[FB] fail to load user information");
	                FB_DispatchEvent("FB_Login", 1, "Fail to load user information");
	            }
	        }
	    });
	    // Execute the batch of requests asynchronously
	    requestBatch.executeAsync();
	}
}

void showDialogWithoutNotificationBar(final String action,final Bundle params) 
{
	s_Activity.runOnUiThread(new Runnable() {
		public void run() {
			s_Activity.dialog = new WebDialog.Builder(s_Activity, Session.getActiveSession(), action, params).setOnCompleteListener(new WebDialog.OnCompleteListener() {
				@Override
				public void onComplete(Bundle values, FacebookException error) {
					if (error != null) {
						LogI("[FB] showDialogWithoutNotificationBar exception " + error);
						FB_DispatchEvent(action, 1, error.toString());
					}
					else
					{
						LogI("[FB] showDialogWithoutNotificationBar sucess ");
						try
						{						
							JSONObject object = new JSONObject();
							for(String key : values.keySet())
							{
								object.put(key,values.getString(key));
							}
							FB_DispatchEvent(action,0,object.toString());			
						}
						catch (Exception e) 
						{
							e.printStackTrace();
							FB_DispatchEvent(action);
						}
					}
					s_Activity.dialog = null;
					s_Activity.dialogAction = null;
					s_Activity.dialogParams = null;
				}
			}).build();

			android.view.Window dialog_window = s_Activity.dialog.getWindow();
			dialog_window.setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
			WindowManager.LayoutParams.FLAG_FULLSCREEN);

			s_Activity.dialogAction = action;
			s_Activity.dialogParams = params;

			s_Activity.dialog.show();
		}
	});
	
}


static String getLoggedInKey() {
	return LOGGED_IN_KEY;
}

static String getCurrentFbUserKey() {
	return CURRENT_FB_USER_KEY;
}

static String getFriendsKey() {
		return FRIENDS_KEY;
}


GraphUser getCurrentFBUser() {
	return currentFBUser;
}

void setCurrentFBUser(GraphUser currentFBUser) {
	this.currentFBUser = currentFBUser;
}

boolean isLoggedIn() {
	return fb_loggedIn;
}

void setLoggedIn(boolean loggedIn) {
	this.fb_loggedIn = loggedIn;
	if (!fb_loggedIn) {
		// If the user is logged out, reset the score and nullify all the logged-in user's values
		setCurrentFBUser(null);
	}
}

ArrayList<String> getFriendsAsArrayListOfStrings() {
	ArrayList<String> friendsAsArrayListOfStrings = new ArrayList<String>();

	Iterator<GraphUser> friendsIterator = friends.iterator();
	while (friendsIterator.hasNext()) {
		friendsAsArrayListOfStrings.add(friendsIterator.next().getInnerJSONObject().toString());
	}

	return friendsAsArrayListOfStrings;
}

GraphUser getFriend(int index) {
	if (friends != null && friends.size() > index) {
		return friends.get(index);
	} else {
		return null;
	}
}

void setFriends(List<GraphUser> friends) {
	this.friends = friends;
}

void setFriends(ArrayList<GraphUser> friends) {
	this.friends = (List<GraphUser>)friends;
}

List<GraphUser> getFriends() {
	return friends;
}

void setHasRequestPublishPermission(boolean value)
{
	this.hasRequestPublishPermission = value;
}

boolean getHasRequestPublishPermission()
{
	return this.hasRequestPublishPermission;
}


void tokenUpdated() 
{
	if (pendingPost) 
	{
		runOnUiThread(new Runnable() {
			public void run() {
				facebookPostAll();
			}
		});
		
	}
}

boolean hasGrantedPublishPermission()
{
	Session session = Session.getActiveSession();
	if (session == null || !session.isOpened()) {
		return false;
	}
	List<String> permissions = session.getPermissions();
	return permissions.containsAll(PERMISSIONS);
}

void requestPublishPermission()
{	
	Session session = Session.getActiveSession();
	if (session == null || !session.isOpened()) {
		return;
	}

	List<String> permissions = session.getPermissions();
	if (!permissions.containsAll(PERMISSIONS)) {		
		Session.NewPermissionsRequest newPermissionsRequest = new Session.NewPermissionsRequest(this, PERMISSIONS)
			.setDefaultAudience(SessionDefaultAudience.FRIENDS);
		session.requestNewPublishPermissions(newPermissionsRequest);
		s_Activity.setHasRequestPublishPermission(true);
		return;
	}
}

void facebookPostAll()
{
	pendingPost = false;
	Session session = Session.getActiveSession();
	if (session == null || !session.isOpened()) {
		return;
	}

	List<String> permissions = session.getPermissions();
	if (!permissions.containsAll(PERMISSIONS)) {
		pendingPost = true;
		Session.NewPermissionsRequest newPermissionsRequest = new Session.NewPermissionsRequest(this, PERMISSIONS)
			.setDefaultAudience(SessionDefaultAudience.FRIENDS);
		session.requestNewPublishPermissions(newPermissionsRequest);
		return;
	}

	fb_PostScore();
}


void fb_PostScore()
{
	LogI("[FB] FB_PostScore - do post");
	Bundle fbParams = new Bundle();
	fbParams.putString("score", "" + mPendingPostScore);
	Request postScoreRequest = new Request(
		Session.getActiveSession(), 
		"me/scores",
		fbParams,
		HttpMethod.POST,
		new Request.Callback() {
			@Override
			public void onCompleted(Response response) {
				FacebookRequestError error = response.getError();
				if (error != null) {
					LogI("[FB] Fail to post score" + error.toString());
					FB_DispatchEvent("FB_PostScore", 1, error.toString());
				} else {
					LogI("[FB] Score posted successfully to Facebook");
					FB_DispatchEvent("FB_PostScore");
				}
			}
		}
	);

	Request.executeBatchAsync(postScoreRequest);	
}


void FB_DispatchEvent(String service)
{
	FB_DispatchEvent(service, 0, "");
}
void FB_DispatchEvent(String service, int errorCode, String msg)
{
	MyActivity.DispatchLua("gGameInstance","bridge_FbEvent", new String[]{"errorCode", errorCode + "", "service", service, "msg", msg});
}


//***************************************************
// FACEBOOK API
//***************************************************


public static void FB_Login()
{
	LogI("[FB] FB_Login");
	Session session = Session.getActiveSession();
	s_Activity.setHasRequestPublishPermission(false);
	if (!session.isOpened() && !session.isClosed()) {
		session.openForRead(new Session.OpenRequest(s_Activity)
			.setCallback(s_Activity.statusCallback)
		);		
	} else {
		Session.openActiveSession(s_Activity, true, s_Activity.statusCallback);
	}
}


public static void FB_Logout()
{
	LogI("[FB] FB_Logout");
	Session session = Session.getActiveSession();
	if (!session.isClosed()) {
		session.closeAndClearTokenInformation();
	}
	s_Activity.setFriends(null);
	s_Activity.setCurrentFBUser(null);
	s_Activity.setCurrentFBUser(null);
	s_Activity.setLoggedIn(false);
}

public static void FB_Feed(String name, String caption, String description, String picture, String link)
{
	LogI("[FB] FB_Feed " + name + " " + caption + " " + description + " " + picture);
	Bundle params = new Bundle();
	params.putString("name", name);
	params.putString("caption", caption);
	params.putString("description", description);
	params.putString("picture", picture);
	params.putString("link", link);
	s_Activity.showDialogWithoutNotificationBar("feed", params);
}

public static void FB_Brag(String name, String caption, String description, String picture)
{
	LogI("[FB] FB_Brag " + name + " " + caption + " " + description + " " + picture);
	Bundle params = new Bundle();
	params.putString("name", name);
	params.putString("caption", caption);
	params.putString("description", description);
	params.putString("picture", picture);
	
	GraphUser currentFBUser = s_Activity.getCurrentFBUser();
	if (currentFBUser != null) 
	{
		String link = "https://apps.facebook.com/" + TOSTRING(FACEBOOK_APP_KEY) + "/?challenge_brag=" + currentFBUser.getId();
		params.putString("link", link);
		LogI("[FB] FB_Brag link: " + link);
	}
	else
	{
		LogI("[FB] FB_Brag - No user detected");
	}

	s_Activity.showDialogWithoutNotificationBar("feed", params);
}


public static void FB_Invite(String message, String data)
{
	LogI("[FB] FB_Invite " + message);
	Bundle params = new Bundle();
	params.putString("message", message);
	params.putString("data", data);
	s_Activity.showDialogWithoutNotificationBar("apprequests", params);
}

public static void FB_DirectRequest(String message, String to, String data) 
{
	Bundle params = new Bundle();
	params.putString("message", message);
	params.putString("to", to);
	params.putString("data", data);
	s_Activity.showDialogWithoutNotificationBar("apprequests", params);
}

public static void FB_SuggestFriends(String message, String friend_id_list, String data) 
{
	Bundle params = new Bundle();
	params.putString("message", message);
	params.putString("suggestions", friend_id_list);
	params.putString("data", data);
	s_Activity.showDialogWithoutNotificationBar("apprequests", params);
}

public static int FB_IsLoggedIn()
{
	return s_Activity.isLoggedIn()?1:0;
}

public static int FB_IsSessionAlive()
{	
	Session session = Session.getActiveSession();	
	return session.isOpened()?1:0;
}

public static void FB_PostScore(String score)
{
	s_Activity.mPendingPostScore = score;
	s_Activity.pendingPost = true;
	s_Activity.tokenUpdated();
}

public static void FB_GetInfo(final String requestURL, final String service)
{
	s_Activity.runOnUiThread(new Runnable() {
		public void run() {
			s_Activity._FB_GetInfo(requestURL, service);
		}
	});
}

public static void FB_Pub(final String requestURL, final String service)
{
	s_Activity.runOnUiThread(new Runnable() {
		public void run() {
			s_Activity._FB_GetInfo(requestURL, service);
		}
	});
}


void _FB_GetInfo(String requestURL,final String service)
{
	Bundle fbParams = new Bundle();
	Request getInfoRequest = new Request(Session.getActiveSession(),
		requestURL,
		fbParams,
		HttpMethod.GET,
		new Request.Callback() {
			@Override
			public void onCompleted(Response response) {
				FacebookRequestError error = response.getError();
				if (error != null)
				{
					MyActivity.DispatchLua("gGameInstance","bridge_FbGetInfo", new String[]{"errorCode","1","service", service});
				}
				else
				{
					GraphObject go = response.getGraphObject();
					JSONObject jso = go.getInnerJSONObject();					
					
					MyActivity.DispatchLua("gGameInstance","bridge_FbGetInfo", new String[]{"errorCode","0", "data", jso.toString(), "service", service});
				}
			}
		});
	Request.executeBatchAsync(getInfoRequest);
}

void _FB_PublishAction(final String appNamespace, final String actionType, final String objectType, final String url, final String title, final String description, final String imageUrl)
{		
	try 
	{
		//Create object
		String objTypeWithNamespace = appNamespace + ":" + objectType;			
		JSONObject object = new JSONObject();
		
		object.put("type", objTypeWithNamespace);
		object.put("url", url);
		object.put("title", title);
		object.put("description", description);
		object.put("image", imageUrl);		
		
		Bundle params = new Bundle();
		params.putString("object", object.toString());	

		Request createObjectRequest = new Request(
			Session.getActiveSession(),
			"me/objects/"+ objTypeWithNamespace,
			params,
			HttpMethod.POST,
			new Request.Callback() {
				@Override
				public void onCompleted(Response response) {
					
					FacebookRequestError error = response.getError();
					if (error != null)
					{
						LogI("---------------Create object failed-------------");
						LogI(error.getErrorMessage());					
					}
					else
					{
						GraphObject go = response.getGraphObject();
						JSONObject jso = go.getInnerJSONObject();					
						try
						{
							String id = (String)jso.get("id");
							LogI("---------------Create object successful-------------");
							LogI(id);
							
							Bundle params = new Bundle();
							params.putString(objectType, id);
							
							Request publishActionRequest = new Request(
								Session.getActiveSession(),
								"me/" + appNamespace + ":" + actionType,
								params,
								HttpMethod.POST,
								new Request.Callback() {
									@Override
									public void onCompleted(Response response) {								
										FacebookRequestError error = response.getError();
										if (error != null)
										{			
											LogI(error.getErrorMessage());		
											MyActivity.DispatchLua("gGameInstance","bridge_FbPublishAction", new String[]{"errorCode","1"});	
										}
										else
										{
											MyActivity.DispatchLua("gGameInstance","bridge_FbPublishAction", new String[]{"errorCode","0"});
										}
									}
								}
							);
							
							Request.executeBatchAsync(publishActionRequest);	
						}
						catch(Exception ex)
						{
							MyActivity.DispatchLua("gGameInstance","bridge_FbPublishAction", new String[]{"errorCode","1"});
						}
					}
				}
			}
		);
		Request.executeBatchAsync(createObjectRequest);
	} 
	catch (Exception e) 
	{
		e.printStackTrace();
	}	
}

public static void FB_PublishAction(final String appNamespace, final String actionType, final String objectType, final String url, final String title, final String description, final String imageUrl)
{	
	s_Activity.runOnUiThread(new Runnable() {
		public void run() {
			s_Activity._FB_PublishAction(appNamespace, actionType, objectType, url, title, description, imageUrl);
		}
	});
}

public static String FB_GetUserInfo()
{
	return String.format("
		{
			\"id\": \"%s\",
			\"name\": \"%s\",
			\"link\": \"%s\"
		}
	",
		s_Activity.currentFBUser.getId(),
		s_Activity.currentFBUser.getName(),
		s_Activity.currentFBUser.getLink()
	);
}

public static void FB_GetRequestInfo(final String requestID)
{
	s_Activity.runOnUiThread(new Runnable() {
		public void run() {
			s_Activity._FB_GetRequestInfo(requestID);
		}
	});
}

void _FB_GetRequestInfo(String requestID)
{
	final Session session = Session.getActiveSession();
	Request requestIDGraphPathRequest = Request.newGraphPathRequest(session, requestID, new Request.Callback() 
	{
		@Override
		public void onCompleted(Response response) 
		{
			FacebookRequestError error = response.getError();
			if (error != null) 
			{
				LogI("[FB] FB_GetRequestInfo error 1 " + error);
				s_Activity.FB_DispatchEvent("FB_GetRequestInfo", 1, error.toString());
			}
			else
			{
				String id = "";
				if (session == Session.getActiveSession()) 
				{
					if (response != null) 
					{
						GraphObject graphObject = response.getGraphObject();
						JSONObject fromObject = (JSONObject)graphObject.getProperty("from");
						try 
						{
							id = fromObject.getString("id");
						} 
						catch (Exception e) 
						{
							LogI("[FB] FB_GetRequestInfo error 2 " + e);
							s_Activity.FB_DispatchEvent("FB_GetRequestInfo", 2, e.toString());
						}
					}
				}
				if (id.compareTo("") != 0)
				{
					LogI("[FB] FB_GetRequestInfo error 3 invalid Request");
					s_Activity.FB_DispatchEvent("FB_GetRequestInfo", 3, "Invalid Request");
				}
				else
				{
					// ask for user info
					Request userGraphPathRequest = Request.newGraphPathRequest(session, id, new Request.Callback() 
					{
						@Override
						public void onCompleted(Response response) 
						{
							FacebookRequestError error = response.getError();
							if (error != null) 
							{
								LogI("[FB] FB_GetRequestInfo error 4 " + error);
								s_Activity.FB_DispatchEvent("FB_GetRequestInfo", 4, error.toString());
							}
							else
							{ 
								boolean isValidRequest = false;
								if (session == Session.getActiveSession()) 
								{
									if (response != null) 
									{
										String uid = response.getGraphObjectAs(GraphUser.class).getId();
										String name = response.getGraphObjectAs(GraphUser.class).getName();
										String link = response.getGraphObjectAs(GraphUser.class).getLink();

										String re = String.format("
											{
												\"id\": \"%s\",
												\"name\": \"%s\",
												\"link\": \"%s\"
											}
										",
											uid,
											name,
											link
										);

										isValidRequest = true;
										s_Activity.FB_DispatchEvent("FB_GetRequestInfo", 0, re);
									}
								}
								if (!isValidRequest)
								{
									LogI("[FB] FB_GetRequestInfo error 5 invalid Request");
									s_Activity.FB_DispatchEvent("FB_GetRequestInfo", 5, "Invalid Request");
								}
							}
						}
					});
					Request.executeBatchAsync(userGraphPathRequest);
					//end of request user info
				}
			}
		}
	});
	Request.executeBatchAsync(requestIDGraphPathRequest);
}

#endif

public static void FB_OpenPageInFacebookApp(final String facebookID)
{	
	Intent facebookAppIntent;
	try 
	{
		Context context = (Context)(s_Activity);
	    context.getPackageManager().getPackageInfo("com.facebook.katana", 0);
	    facebookAppIntent = new Intent(Intent.ACTION_VIEW, Uri.parse("fb://page/" + facebookID));
	} 
	catch (Exception e) 
	{
	    facebookAppIntent = new Intent(Intent.ACTION_VIEW, Uri.parse("https://www.facebook.com/" + facebookID));
	}

	if (facebookAppIntent != null)
	{
		facebookAppIntent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_WHEN_TASK_RESET);
		s_Activity.startActivity(facebookAppIntent);
	}
}

