#if USE_WIFI_DIRECT
/********************************************************************
Bluetooth interface
/********************************************************************/
static boolean IsSupportWifiDirect()
{
	return (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.ICE_CREAM_SANDWICH);
}

static void WifiDirect_Init()
{
	LogI("[WifiDirect] WifiDirect_Init()");
	s_Activity.SendMessge(MESSAGE_WD_INIT);	
}

static void WifiDirect_OpenSetting()
{
	LogI("[WifiDirect] WifiDirect_OpenSetting()");
	s_Activity.SendMessge(MESSAGE_WD_SETTING);	
}

static String WifiDirect_GetDeviceName()
{
	LogI("[WifiDirect] WifiDirect_GetDeviceName()");
	return s_Activity.GetWifiDirectDeviceName();
}

static void WifiDirectDriver_SearchDevice()
{
	LogI("[WifiDirect] WifiDirectDriver_SearchDevice()");
	s_Activity.SendMessge(MESSAGE_WD_SEARCH_DEVICE);	
}

static void WifiDirect_Connect(String addr)
{
	LogI("[WifiDirect] WifiDirect_Connect()" + addr);
	s_Activity.SendMessge(MESSAGE_WD_CONNECT, addr);	
}

static void WifiDirectDriver_CancelConnection()
{
	LogI("[WifiDirect] WifiDirectDriver_CancelConnection()");
	s_Activity.SendMessge(MESSAGE_WD_CANCEL_CONNECT);	
}

private final IntentFilter wifiDirect_intentFilter = new IntentFilter();
private WifiP2pManager manager;
private Channel channel;
private BroadcastReceiver wifiDirect_receiver = null;
private List peers = new ArrayList();
private PeerListListener peerListListener;


private String mWifiDirectHostIP = "";
public void SetHostIP(String ip)
{
	mWifiDirectHostIP = ip;
}
public String GetHostIP()
{
	return mWifiDirectHostIP;
}

private boolean mIsWifiDirectHost = false;
public void SetHost(boolean isHost)
{
	mIsWifiDirectHost = isHost;
}
public boolean IsHost()
{
	return mIsWifiDirectHost;
}

private boolean m_IsWifiP2pEnabled = false;
public void SetIsWifiP2pEnabled(boolean isWifiP2pEnabled) 
{
	m_IsWifiP2pEnabled = isWifiP2pEnabled;
}
public boolean IsWifiP2pEnabled()
{
	return m_IsWifiP2pEnabled;
}

private String mWifiDirectDeviceName = "unknown";
public void SetWifiDirectDeviceName(String name)
{
	mWifiDirectDeviceName = name;
}
public String GetWifiDirectDeviceName()
{
	return mWifiDirectDeviceName;
}



//*********************************************************************************************************************************
private void _WifiDirect_Init()
{
	if (!IsSupportWifiDirect())
	{
		MyActivity.DispatchLua("gWifiDirectDriver","bridge_WifiDirect_Init_Completed", new String[]{"errorCode", "1", "msg", "Wifi Direct is not supported"});
	}
	else if (!m_IsWifiP2pEnabled)
	{
		MyActivity.DispatchLua("gWifiDirectDriver","bridge_WifiDirect_Init_Completed", new String[]{"errorCode", "2", "msg", "Wifi Direct is not enabled"});
	}
	else
	{
		manager.removeGroup(channel, new ActionListener() {
			@Override
			public void onFailure(int reasonCode) {
				LogI("WifiDirect Remove Group fail (init)");
			}

			@Override
			public void onSuccess() {
				LogI("WifiDirect Remove Group success (init)");
			}
		});
	
		manager.discoverPeers(channel, null); //make sure this peers is discoverable
		MyActivity.DispatchLua("gWifiDirectDriver","bridge_WifiDirect_Init_Completed", 
			new String[]{
				"errorCode", "0", 
				"msg", "Enable Wifi Direct successfully",
			}
		);
	}
}

private void _WifiDirect_SearchDevice()
{
	if (!IsSupportWifiDirect())
	{
		LogI("_WifiDirect_SearchDevice: wifiDirect is not supported");
		MyActivity.DispatchLua("gWifiDirectDriver","bridge_WifiDirect_Discovered_Devices", new String[]{"errorCode", "1", "devices", ""});
		return;
	}
	
	if (!m_IsWifiP2pEnabled)
	{
		LogI("_WifiDirect_SearchDevice: wifiDirect is not inited");
		MyActivity.DispatchLua("gWifiDirectDriver","bridge_WifiDirect_Discovered_Devices", new String[]{"errorCode", "2", "devices", ""});
		return;
	}
	
	manager.discoverPeers(channel, new WifiP2pManager.ActionListener() 
		{
			@Override
			public void onSuccess() 
			{
				LogI("_WifiDirect_SearchDevice: Discovery Initiated");
				manager.requestPeers(channel, new PeerListListener() {
					@Override
					public void onPeersAvailable(WifiP2pDeviceList peerList) 
					{
						// Out with the old, in with the new.
						peers.clear();
						peers.addAll(peerList.getDeviceList());

						if (peers.size() == 0) 
						{
							LogI("PeerListListener: No devices found");
							MyActivity.DispatchLua("gWifiDirectDriver","bridge_WifiDirect_Discovered_Devices", new String[]{"errorCode", "0", "devices", ""});
							return;
						}
						else
						{
							String devices = "";
							for (Object p : peers) {
								WifiP2pDevice peer = (WifiP2pDevice)p;
								LogI(peer.deviceName);
								devices += peer.deviceName + "\n" + peer.deviceAddress + "\n";
							} 
							MyActivity.DispatchLua("gWifiDirectDriver","bridge_WifiDirect_Discovered_Devices", new String[]{"errorCode", "0", "devices", devices});
						}
					}
				});
			}

			@Override
			public void onFailure(int reasonCode) 
			{
				LogI("_WifiDirect_SearchDevice: Discovery Failed " + reasonCode);
				int error = -9999;
				if (reasonCode == WifiP2pManager.P2P_UNSUPPORTED)
				{
					error = -1;
				}
				else if (reasonCode == WifiP2pManager.ERROR)
				{
					error = -2;
				}
				else if (reasonCode == WifiP2pManager.BUSY)
				{
					error = -3;
				}
				
				MyActivity.DispatchLua("gWifiDirectDriver","bridge_WifiDirect_Discovered_Devices", new String[]{"errorCode", error+"", "devices", ""});
			}
		}
	);
}

private void _WifiDirect_Connect(String address)
{
	if (!IsSupportWifiDirect())
	{
		LogI("_WifiDirect_Connect: wifiDirect is not supported");
		MyActivity.DispatchLua("gWifiDirectDriver","bridge_WifiDirect_Connected_Fail");
		return;
	}
	
	WifiP2pConfig config = new WifiP2pConfig();
	config.deviceAddress = address;
	config.wps.setup = WpsInfo.PBC;

	manager.connect(channel, config, new ActionListener() 
		{
			@Override
			public void onSuccess() 
			{
				LogI("WiFiDirectBroadcastReceiver: connect onSuccess");
			}

			@Override
			public void onFailure(int reason) 
			{
				LogI("WiFiDirectBroadcastReceiver: connect onFailure");
				MyActivity.DispatchLua("gWifiDirectDriver","bridge_WifiDirect_Connected_Fail");
			}
		}
	);
}

private void _WifiDirect_CancelConnection()
{
	if (IsSupportWifiDirect())
	{
		manager.cancelConnect (channel, null);
		manager.removeGroup(channel, null);
	}
}


//*********************************************************************************************************************************

private void WifiDirect_HandleMessage(Message msg)
{
	if (msg.what >= MESSAGE_WD_INIT && msg.what <= MESSAGE_WD_CANCEL_CONNECT)
	{
		LogI("[WifiDirect] WifiDirect_HandleMessage() " + msg.what);
	}
	switch (msg.what) 
	{
		case MESSAGE_WD_INIT:
			_WifiDirect_Init();
			break;
			
		case MESSAGE_WD_SETTING:
			startActivity(new Intent(android.provider.Settings.ACTION_WIRELESS_SETTINGS));
			break;
		
		case MESSAGE_WD_SEARCH_DEVICE:
			_WifiDirect_SearchDevice();
			break;
			
		case MESSAGE_WD_CONNECT:
			String addr = (String)msg.obj;
			_WifiDirect_Connect(addr);
			break;
			
		case MESSAGE_WD_CANCEL_CONNECT:
			_WifiDirect_CancelConnection();
			break;
			
	}
}

//*********************************************************************************************************************************

void WifiDirect_OnCreate()
{
	LogI("WifiDirect_OnCreate");
	
	if (!IsSupportWifiDirect()) {return;}
	
    wifiDirect_intentFilter.addAction(WifiP2pManager.WIFI_P2P_STATE_CHANGED_ACTION);
	wifiDirect_intentFilter.addAction(WifiP2pManager.WIFI_P2P_CONNECTION_CHANGED_ACTION);
	wifiDirect_intentFilter.addAction(WifiP2pManager.WIFI_P2P_THIS_DEVICE_CHANGED_ACTION);
	
	
	manager = (WifiP2pManager) getSystemService(Context.WIFI_P2P_SERVICE);
	channel = manager.initialize(this, getMainLooper(), null);
	
	manager.removeGroup(channel, new ActionListener() {
		@Override
		public void onFailure(int reasonCode) {
			LogI("WifiDirect Remove Group fail");
		}

		@Override
		public void onSuccess() {
			LogI("WifiDirect Remove Group success");
		}
	});
	
	wifiDirect_receiver = new WiFiDirectBroadcastReceiver(manager, channel, this);
}

void WifiDirect_onResume() 
{
	LogI("WifiDirect_onResume");
	if (!IsSupportWifiDirect()) {return;}
	registerReceiver(wifiDirect_receiver, wifiDirect_intentFilter);
}

void WifiDirect_onPause() 
{
	LogI("WifiDirect_onPause");
	if (!IsSupportWifiDirect()) {return;}
	try
	{
		unregisterReceiver(wifiDirect_receiver);
	}
	catch (Exception E) {}
}

void WifiDirect_onDestroy() 
{
	if (!IsSupportWifiDirect()) {return;}
	try
	{
		unregisterReceiver(wifiDirect_receiver);
	}
	catch (Exception E) {}
}






#endif
















