static int Chartboost_Show()
{
#if USE_CHARTBOOST
	if (!s_Activity.cb.hasCachedInterstitial())
	{
		LogI("[Chartboost_Show] not cache yet -> do cache");
		Chartboost_Cache();
		return 0;
	}
	else
	{
		s_Activity.runOnUiThread(new Runnable() {
			public void run() {
				s_Activity.cb.showInterstitial();
			}
		});
		return 1;
	}
#else
	LogI("[Chartboost_Show] USE_CHARTBOOST is 0");
	return 0;
#endif
}

static void Chartboost_Cache()
{
#if USE_CHARTBOOST
	s_Activity.runOnUiThread(new Runnable() {
		public void run() {
			s_Activity.cb.cacheInterstitial();
		}
	});
#else
	LogI("[Chartboost_Cache] USE_CHARTBOOST is 0");
#endif
}

#if USE_CHARTBOOST

	private Chartboost cb;

	private void Chartboost_OnCreate()
	{
		LogI("[Chartboost_OnCreate] id = " + TOSTRING(USE_CHARTBOOST_APP_ID) + " sign = " + TOSTRING(USE_CHARTBOOST_APP_SIGN));
		this.cb = Chartboost.sharedChartboost();
		String appId = TOSTRING(USE_CHARTBOOST_APP_ID);
		String appSignature = TOSTRING(USE_CHARTBOOST_APP_SIGN);
		this.cb.onCreate(this, appId, appSignature, chartBoostDelegate);
		this.cb.startSession();
	}
	
	private void Chartboost_OnStart()
	{
		LogI("Chartboost_OnStart");
		this.cb.onStart(this);
	}
	
	private void Chartboost_OnStop()
	{
		LogI("Chartboost_OnStop");
		this.cb.onStop(this);
	}
	
	private void Chartboost_OnDestroy()
	{
		LogI("Chartboost_OnDestroy");
		this.cb.onDestroy(this);
	}
	
	private boolean Chartboost_OnBack()
	{
		LogI("Chartboost_OnBack");
		return this.cb.onBackPressed();
	}
	
	private ChartboostDelegate chartBoostDelegate = new ChartboostDelegate() {
		/* 
		 * shouldDisplayInterstitial(String location)
		 *
		 * This is used to control when an interstitial should or should not be displayed
		 * If you should not display an interstitial, return FALSE
		 *
		 * For example: during gameplay, return FALSE.
		 *
		 * Is fired on:
		 * - showInterstitial()
		 * - Interstitial is loaded & ready to display
		 */
		@Override
		public boolean shouldDisplayInterstitial(String location) {
			return true;
		}

		/*
		 * shouldRequestInterstitial(String location)
		 * 
		 * This is used to control when an interstitial should or should not be requested
		 * If you should not request an interstitial from the server, return FALSE
		 *
		 * For example: user should not see interstitials for some reason, return FALSE.
		 *
		 * Is fired on:
		 * - cacheInterstitial()
		 * - showInterstitial() if no interstitial is cached
		 * 
		 * Notes: 
		 * - We do not recommend excluding purchasers with this delegate method
		 * - Instead, use an exclusion list on your campaign so you can control it on the fly
		 */
		@Override
		public boolean shouldRequestInterstitial(String location) {
			return true;
		}

		/*
		 * didCacheInterstitial(String location)
		 * 
		 * Passes in the location name that has successfully been cached
		 * 
		 * Is fired on:
		 * - cacheInterstitial() success
		 * - All assets are loaded
		 * 
		 * Notes:
		 * - Similar to this is: cb.hasCachedInterstitial(String location) 
		 * Which will return true if a cached interstitial exists for that location
		 */
		@Override
		public void didCacheInterstitial(String location) {
		   // Save which location is ready to display immediately
		   LogI("[Chartboost] didCacheInterstitial");
		}

		/*
		 * didFailToLoadInterstitial(String location)
		 * 
		 * This is called when an interstitial has failed to load for any reason
		 * 
		 * Is fired on:
		 * - cacheInterstitial() failure
		 * - showInterstitial() failure if no interstitial was cached
		 * 
		 * Possible reasons:
		 * - No network connection
		 * - No publishing campaign matches for this user (go make a new one in the dashboard)
		 */
		@Override
		public void didFailToLoadInterstitial(java.lang.String location, com.chartboost.sdk.Model.CBError.CBImpressionError p) {
			LogI("[Chartboost] didFailToLoadInterstitial " + location);
			// Show a house ad or do something else when a chartboost interstitial fails to load
		}

		/*
		 * didDismissInterstitial(String location)
		 *
		 * This is called when an interstitial is dismissed
		 *
		 * Is fired on:
		 * - Interstitial click
		 * - Interstitial close
		 *
		 * #Pro Tip: Use the code below to immediately re-cache interstitials
		 */
		@Override
		public void didDismissInterstitial(String location) {
			LogI("[Chartboost] didDismissInterstitial " + location);
			// Immediately re-caches an interstitial
			cb.cacheInterstitial(location);
		}

		/*
		 * didCloseInterstitial(String location)
		 *
		 * This is called when an interstitial is closed
		 *
		 * Is fired on:
		 * - Interstitial close
		 */
		@Override
		public void didCloseInterstitial(String location) {
			// Know that the user has closed the interstitial
			LogI("[Chartboost] didCloseInterstitial " + location);
		}

		/*
		 * didClickInterstitial(String location)
		 *
		 * This is called when an interstitial is clicked
		 *
		 * Is fired on:
		 * - Interstitial click
		 */
		@Override
		public void didClickInterstitial(String location) {
			// Know that the user has clicked the interstitial
			LogI("[Chartboost] didClickInterstitial " + location);
		}

		/*
		 * didShowInterstitial(String location)
		 *
		 * This is called when an interstitial has been successfully shown
		 *
		 * Is fired on:
		 * - showInterstitial() success
		 */
		@Override
		public void didShowInterstitial(String location) {
			// Know that the user has seen the interstitial
			LogI("[Chartboost] didShowInterstitial " + location);
		}

		/*
		 * More Apps delegate methods
		 */

		/*
		 * shouldDisplayLoadingViewForMoreApps()
		 *
		 * Return FALSE to prevent the pretty More-Apps loading screen
		 *
		 * Is fired on:
		 * - showMoreApps()
		 */
		@Override
		public boolean shouldDisplayLoadingViewForMoreApps() {
			return true;
		}

		/*
		 * shouldRequestMoreApps()
		 * 
		 * Return FALSE to prevent a More-Apps page request
		 *
		 * Is fired on:
		 * - cacheMoreApps()
		 * - showMoreApps() if no More-Apps page is cached
		 */
		@Override
		public boolean shouldRequestMoreApps() {
			return true;
		}

		/*
		 * shouldDisplayMoreApps()
		 * 
		 * Return FALSE to prevent the More-Apps page from displaying
		 *
		 * Is fired on:
		 * - showMoreApps() 
		 * - More-Apps page is loaded & ready to display
		 */
		@Override
		public boolean shouldDisplayMoreApps() {
			return true;
		}

		/*
		 * didFailToLoadMoreApps()
		 * 
		 * This is called when the More-Apps page has failed to load for any reason
		 * 
		 * Is fired on:
		 * - cacheMoreApps() failure
		 * - showMoreApps() failure if no More-Apps page was cached
		 * 
		 * Possible reasons:
		 * - No network connection
		 * - No publishing campaign matches for this user (go make a new one in the dashboard)
		 */
		@Override
		public void didFailToLoadMoreApps(com.chartboost.sdk.Model.CBError.CBImpressionError p) {
			// Do something else when the More-Apps page fails to load
		}

		/*
		 * didCacheMoreApps()
		 * 
		 * Is fired on:
		 * - cacheMoreApps() success
		 * - All assets are loaded
		 */
		@Override
		public void didCacheMoreApps() {
			// Know that the More-Apps page is cached and ready to display
		}

		/*
		 * didDismissMoreApps()
		 *
		 * This is called when the More-Apps page is dismissed
		 *
		 * Is fired on:
		 * - More-Apps click
		 * - More-Apps close
		 */
		@Override
		public void didDismissMoreApps() {
			// Know that the More-Apps page has been dismissed
		}

		/*
		 * didCloseMoreApps()
		 *
		 * This is called when the More-Apps page is closed
		 *
		 * Is fired on:
		 * - More-Apps close
		 */
		@Override
		public void didCloseMoreApps() {
			// Know that the More-Apps page has been closed
		}

		/*
		 * didClickMoreApps()
		 *
		 * This is called when the More-Apps page is clicked
		 *
		 * Is fired on:
		 * - More-Apps click
		 */
		@Override
		public void didClickMoreApps() {
			// Know that the More-Apps page has been clicked

		}

		/*
		 * didShowMoreApps()
		 *
		 * This is called when the More-Apps page has been successfully shown
		 *
		 * Is fired on:
		 * - showMoreApps() success
		 */
		@Override
		public void didShowMoreApps() {
			// Know that the More-Apps page has been presented on the screen
		}

		/*
		 * shouldRequestInterstitialsInFirstSession()
		 *
		 * Return FALSE if the user should not request interstitials until the 2nd startSession()
		 * 
		 */
		@Override
		public boolean shouldRequestInterstitialsInFirstSession() {
			return true;
		}
		/*
		*  didFailToLoadURL(String url)
		*  
		*  This is called when a URL fails to load (for any reason) after a user clicks an ad.
		*  
		*  Is fired on:
		*      - Interstitial click
		*      - More Apps click
		*   
		*  Possible reasons:
		*      - No network connection
		*      - No valid activity to launch
		*      - Unable to parse URL        
		*  
		*/
		// @Override
		// public void didFailToLoadUrl(String url) {
			// Know that the attempt to load the URL has failed
		// }

		@Override
		public boolean shouldPauseClickForConfirmation(com.chartboost.sdk.Chartboost.CBAgeGateConfirmation p) {
			return true;
		}

		@Override
		public void didFailToRecordClick(java.lang.String s,com.chartboost.sdk.Model.CBError.CBClickError c) {

		}
	};
	
#endif