#if USE_GOOGLE_PLAY_SERVICE

private final static int PLAY_SERVICES_RESOLUTION_REQUEST = 9000;
public static final String EXTRA_MESSAGE = "message";
public static final String PROPERTY_REG_ID = "googleplay_service_registration_id";
private static final String PROPERTY_APP_VERSION = "appVersion";
private String GCM_SENDER_ID = "442046961296";

private boolean m_IsGPServiceSupported = false;
GoogleCloudMessaging gcm;
AtomicInteger msgId = new AtomicInteger();
SharedPreferences prefs;
String regid;

private static int getAppVersion(Context context) {
	try {
		PackageInfo packageInfo = context.getPackageManager()
				.getPackageInfo(context.getPackageName(), 0);
		return packageInfo.versionCode;
	} catch (NameNotFoundException e) {
		// should never happen
		throw new RuntimeException("Could not get package name: " + e);
	}
}

private boolean checkGooglePlayServices() {
	int resultCode = GooglePlayServicesUtil.isGooglePlayServicesAvailable(this);
	if (resultCode != ConnectionResult.SUCCESS) {
		if (GooglePlayServicesUtil.isUserRecoverableError(resultCode)) {
			GooglePlayServicesUtil.getErrorDialog(resultCode, this,
					PLAY_SERVICES_RESOLUTION_REQUEST).show();
		} else {
			LogI("[GooglePlayService] [checkGooglePlayServices] This device is not supported.");
			m_IsGPServiceSupported = false;
		}
		m_IsGPServiceSupported = false;
	}
	m_IsGPServiceSupported = true;
	return m_IsGPServiceSupported;
}

private SharedPreferences getGCMPreferences(Context context) {
	// This sample app persists the registration ID in shared preferences, but
	// how you store the regID in your app is up to you.
	return getSharedPreferences(MyActivity.class.getSimpleName(),
			Context.MODE_PRIVATE);
}

private String getRegistrationId(Context context) {
	final SharedPreferences prefs = getGCMPreferences(context);
	String registrationId = prefs.getString(PROPERTY_REG_ID, "");
	try {
		if (registrationId.isEmpty()) {
			LogI("[GCM] [getRegistrationId] Registration not found.");
			return "";
		}
	}
	catch (Exception E) 
	{
		if (registrationId == null || registrationId.length() == 0) { //android 2.2 compat
			LogI("[GCM] [getRegistrationId] Registration not found.");
			return "";
		}
	}
	// Check if app was updated; if so, it must clear the registration ID
	// since the existing regID is not guaranteed to work with the new
	// app version.
	int registeredVersion = prefs.getInt(PROPERTY_APP_VERSION, Integer.MIN_VALUE);
	int currentVersion = getAppVersion(context);
	if (registeredVersion != currentVersion) {
		LogI("[GCM] [getRegistrationId] App version changed.");
		return "";
	}
	return registrationId;
}

private void registerInBackground(final Context context) {
	new AsyncTask() {
		@Override
		protected String doInBackground(Object... params) {
			try {
				if (gcm == null) {
					gcm = GoogleCloudMessaging.getInstance(context);
				}
				regid = gcm.register(GCM_SENDER_ID);
				sendRegistrationIdToBackend();
				storeRegistrationId(context, regid);
			} catch (IOException ex) {
				ex.printStackTrace();
				LogI("[GCM] " + ex.toString());
			}
			return "";
		}
	}.execute(null, null, null);
}

private void sendRegistrationIdToBackend() {
	LogI("[GCM] Successfully register GCM with id " + regid);

	MyActivity.DispatchLua("gGameInstance","bridge_GCM_RegisterSuccess", new String[]{
		"id", regid
	});
}

private void storeRegistrationId(Context context, String regId) {
	final SharedPreferences prefs = getGCMPreferences(context);
	int appVersion = getAppVersion(context);
	LogI("[GCM] [storeRegistrationId] Saving regId on app version " + appVersion);
	SharedPreferences.Editor editor = prefs.edit();
	editor.putString(PROPERTY_REG_ID, regId);
	editor.putInt(PROPERTY_APP_VERSION, appVersion);
	editor.commit();
}

private void _GCM_OnCreate()
{
	if (checkGooglePlayServices()) 
	{
		gcm = GoogleCloudMessaging.getInstance(this);
		regid = getRegistrationId(MyActivity.s_Activity);

		if (regid.isEmpty()) {
			registerInBackground(MyActivity.s_Activity);
		}
		else {
			sendRegistrationIdToBackend();
		}
	}
}

public static void GCM_OnCreate()
{
	s_Activity.runOnUiThread(new Runnable() {
		public void run() {
			MyActivity.s_Activity._GCM_OnCreate();
		}
	});
}

#endif



